<?php

namespace Drupal\migrate_wizard\EventSubscriber;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\migrate\Event\MigrateEvents;
use Drupal\migrate\Event\MigratePostRowSaveEvent;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
 * This class is executed after row is saved.
 */
class MWParagraphByLanguageSubscriber implements EventSubscriberInterface {

    /**
   * The entity type manager.
   *
   * @var Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;
  protected $configFactory;

  /**
   * Constructs a new migration subscriber.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity field manager service.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager, ConfigFactoryInterface $config_factory) {
    $this->entityTypeManager = $entity_type_manager;
    $this->configFactory = $config_factory;
  }

  /**
   * Set the CAS user name to the user name entity after migrate.
   */
  public function setLangcodePagagraph(MigratePostRowSaveEvent $event) {
    if($event->getRow()->getSourceProperty('translations')){

      $config = $this->configFactory->getEditable('migrate_wizard.settings');
      $destination_config = $event->getMigration()->getDestinationConfiguration();
      $currentConfig = $config->get($destination_config['default_bundle']);
      foreach(reset($currentConfig) as $field_migration){
        if($field_migration['destiny_type'] == 'paragraph'){
          $destinies = $event->getRow()->getDestinationProperty($field_migration['destiny']);
          foreach($destinies as $delta => $destiny){
            $paragraph = $this->entityTypeManager->getStorage('paragraph')->load($destiny['target_id']);
            $paragraph->set('langcode', $event->getRow()->getDestinationProperty('langcode'));
            $paragraph->save();
          }
        }
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    $events = [];
    if (\Drupal::hasService('cas.user_manager')) {
      $events[MigrateEvents::POST_ROW_SAVE][] = ['setLangcodePagagraph', 0];
    }
    return $events;
  }
}
